@@ -17,8 +17,10 @@
 """ Wrapper of couchdbkit Document and Properties for django. It also
 add possibility to a document to register itself in CouchdbkitHandler
 """
+from __future__ import absolute_import
 import re
 import sys
+import six
 
 try:
     from django.db.models.options import get_verbose_name
@@ -26,9 +28,9 @@ except ImportError:
     from django.utils.text import camel_case_to_spaces as get_verbose_name
 
 from django.conf import settings
-from django.utils.translation import activate, deactivate_all, get_language, \
-string_concat
-from django.utils.encoding import smart_str, force_unicode
+from django.utils.text import format_lazy
+from django.utils.translation import activate, deactivate_all, get_language
+from django.utils.encoding import smart_str, force_text
 
 from couchdbkit import schema
 from couchdbkit.ext.django.loading import get_schema, register_schema, \
@@ -86,13 +88,14 @@ class Options(object):
 
             # verbose_name_plural is a special case because it uses a 's'
             # by default.
-            setattr(self, 'verbose_name_plural', meta_attrs.pop('verbose_name_plural', string_concat(self.verbose_name, 's')))
+            setattr(self, 'verbose_name_plural', meta_attrs.pop(
+                'verbose_name_plural', format_lazy('{}{}', self.verbose_name, 's')))
 
             # Any leftover attributes must be invalid.
             if meta_attrs != {}:
-                raise TypeError("'class Meta' got invalid attribute(s): %s" % ','.join(meta_attrs.keys()))
+                raise TypeError("'class Meta' got invalid attribute(s): %s" % ','.join(list(meta_attrs.keys())))
         else:
-            self.verbose_name_plural = string_concat(self.verbose_name, 's')
+            self.verbose_name_plural = format_lazy('{}{}', self.verbose_name, 's')
         del self.meta
 
     def __str__(self):
@@ -106,7 +109,7 @@ class Options(object):
         """
         lang = get_language()
         deactivate_all()
-        raw = force_unicode(self.verbose_name)
+        raw = force_text(self.verbose_name)
         activate(lang)
         return raw
     verbose_name_raw = property(verbose_name_raw)
@@ -145,9 +148,8 @@ class DocumentMeta(schema.SchemaProperties):
         else:
             setattr(cls, name, value)
 
-class Document(schema.Document):
+class Document(six.with_metaclass(DocumentMeta, schema.Document)):
     """ Document object for django extension """
-    __metaclass__ = DocumentMeta
 
     get_id = property(lambda self: self['_id'])
     get_rev = property(lambda self: self['_rev'])
