@@ -2,9 +2,11 @@
 #
 # This file is part of couchdbkit released under the MIT license.
 # See the NOTICE for more information.
+from __future__ import absolute_import
 import functools
 from jsonobject.properties import *
 from jsonobject.base import DefaultProperty
+import six
 
 try:
     from collections import MutableSet, Iterable
@@ -16,8 +18,8 @@ try:
 except ImportError:
     support_setproperty = False
 
-StringListProperty = functools.partial(ListProperty, unicode)
-StringDictProperty = functools.partial(DictProperty, unicode)
+StringListProperty = functools.partial(ListProperty, six.text_type)
+StringDictProperty = functools.partial(DictProperty, six.text_type)
 
 
 class Property(DefaultProperty):
