@@ -10,13 +10,16 @@ really belong anywhere else in the modules.
 """
 from __future__ import with_statement
 
+from __future__ import absolute_import
+from __future__ import print_function
 import codecs
 import string
 from hashlib import md5
 import os
 import re
 import sys
-import urllib
+import six
+from six.moves import range
 
 
 try:
@@ -123,15 +126,15 @@ def validate_dbname(name):
     """ validate dbname """
     if name in SPECIAL_DBS:
         return True
-    elif not VALID_DB_NAME.match(urllib.unquote(name)):
+    elif not VALID_DB_NAME.match(six.moves.urllib.parse.unquote(name)):
         raise ValueError("Invalid db name: '%s'" % name)
     return True
 
 def to_bytestring(s):
     """ convert to bytestring an unicode """
-    if not isinstance(s, basestring):
+    if not isinstance(s, six.string_types):
         return s
-    if isinstance(s, unicode):
+    if isinstance(s, six.text_type):
         return s.encode('utf-8')
     else:
         return s
@@ -195,7 +198,7 @@ def read_json(filename, use_environment=False):
     """
     try:
         data = read_file(filename, force_read=True)
-    except IOError, e:
+    except IOError as e:
         if e[0] == 2:
             return {}
         raise
@@ -206,7 +209,7 @@ def read_json(filename, use_environment=False):
     try:
         data = json.loads(data)
     except ValueError:
-        print >>sys.stderr, "Json is invalid, can't load %s" % filename
+        print("Json is invalid, can't load %s" % filename, file=sys.stderr)
         raise
     return data
 
@@ -229,3 +232,13 @@ class ProxyDict(jsonobject.utils.SimpleDict):
 
     def __copy__(self):
         return self.copy()
+
+
+# From https://benoitc.github.io/restkit/api/restkit.util-pysrc.html#url_quote
+def url_quote(s, safe='/:'):
+    """URL encode a single string with a given encoding."""
+    if isinstance(s, six.text_type):
+        s = s.encode('utf-8')
+    elif not isinstance(s, six.binary_type):
+        s = six.binary_type(s)
+    return six.moves.urllib.parse.quote(s, safe=safe)
