@@ -3,5 +3,6 @@
 # This file is part of couchdbkit released under the MIT license.
 # See the NOTICE for more information.
 
-version_info = (0, 7, 4, 2)
-__version__ =  ".".join(map(str, version_info))
+from __future__ import absolute_import
+version_info = (1, 0, 1)
+__version__ = ".".join([str(vi) for vi in version_info])
