@@ -3,18 +3,22 @@
 # This file is part of couchdbkit released under the MIT license.
 # See the NOTICE for more information.
 
+from __future__ import absolute_import
+from __future__ import print_function
+import six
 __author__ = 'benoitc@e-engura.com (Benoît Chesneau)'
 
 import datetime
 import decimal
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
+import unittest
 
 from couchdbkit import *
 from couchdbkit.schema.properties import support_setproperty
 
+if six.PY3:
+    unittest.TestCase.assertItemsEqual = unittest.TestCase.assertCountEqual
+
+
 class DocumentTestCase(unittest.TestCase):
     def setUp(self):
         self.server = Server()
@@ -43,7 +47,7 @@ class DocumentTestCase(unittest.TestCase):
         doc.foo="test"
         try:
             doc.bar="bla"
-        except AttributeError, e:
+        except AttributeError as e:
             self.assertEqual(str(e), "'bar' is not defined in schema (not a valid property)")
         doc.save()
         self.assert_(not hasattr(doc, "bar"))
@@ -58,7 +62,7 @@ class DocumentTestCase(unittest.TestCase):
         doc.foo="test"
         try:
             doc.bar="bla"
-        except AttributeError, e:
+        except AttributeError as e:
             self.assertEqual(str(e), "'bar' is not defined in schema (not a valid property)")
         doc.save()
         self.assert_(not hasattr(doc, "bar"))
@@ -180,14 +184,14 @@ class DocumentTestCase(unittest.TestCase):
 
         try:
             Test.bulk_save( [doc1, doc2, doc3] )
-        except TypeError, e:
+        except TypeError as e:
             self.assert_(str(e)== "doc database required to save document" )
 
         Test.set_db( db )
         bad_doc = Test2(string="bad_doc")
         try:
             Test.bulk_save( [doc1, doc2, doc3, bad_doc] )
-        except ValueError, e:
+        except ValueError as e:
             self.assert_(str(e) == "one of your documents does not have the correct type" )
 
         Test.bulk_save( [doc1, doc2, doc3] )
@@ -254,7 +258,7 @@ class DocumentTestCase(unittest.TestCase):
         doc1 = Test.get(doc._id)
         self.server.delete_db('couchdbkit_test')
 
-        self.assert_(isinstance(doc1.field, basestring))
+        self.assert_(isinstance(doc1.field, six.string_types))
         self.assert_(isinstance(doc1.field1, datetime.datetime))
         self.assert_(isinstance(doc1.field2, datetime.date))
         self.assert_(isinstance(doc1.field3, datetime.time))
@@ -425,7 +429,7 @@ class DocumentTestCase(unittest.TestCase):
         results3 = TestDoc.view('test/all', include_docs=True,
                                 wrapper=lambda row: row['doc']['field1'])
         self.assert_(len(results3) == 2)
-        self.assert_(isinstance(results3.first(), unicode) == True)
+        self.assert_(isinstance(results3.first(), six.text_type) == True)
         self.server.delete_db('couchdbkit_test')
 
     def test_wrong_doc_type(self):
@@ -530,31 +534,6 @@ class DocumentTestCase(unittest.TestCase):
         self.assert_(len(results) == 2)
         self.server.delete_db('couchdbkit_test')
 
-
-    def testTempView(self):
-        class TestDoc(Document):
-            field1 = StringProperty()
-            field2 = StringProperty()
-
-        design_doc = {
-            "map": """function(doc) { if (doc.doc_type == "TestDoc") { emit(doc._id, doc);
-}}"""
-        }
-
-        doc = TestDoc(field1="a", field2="b")
-        doc1 = TestDoc(field1="c", field2="d")
-
-        db = self.server.create_db('couchdbkit_test')
-        TestDoc._db = db
-
-        doc.save()
-        doc1.save()
-        results = TestDoc.temp_view(design_doc)
-        self.assert_(len(results) == 2)
-        doc3 = list(results)[0]
-        self.assert_(hasattr(doc3, "field1"))
-        self.server.delete_db('couchdbkit_test')
-
     def testDocumentAttachments(self):
         db = self.server.create_db('couchdbkit_test')
 
@@ -569,12 +548,10 @@ class DocumentTestCase(unittest.TestCase):
         text_attachment = u"un texte attaché"
         old_rev = a._rev
 
-        a.put_attachment(text_attachment, "test", "text/plain")
-        self.assert_(old_rev != a._rev)
-        fetch_attachment = a.fetch_attachment("test")
-        self.assert_(text_attachment == fetch_attachment)
-        self.server.delete_db('couchdbkit_test')
-
+        with self.assertRaises(NoLongerSupportedException):
+            a.put_attachment(text_attachment, "test", "text/plain")
+        with self.assertRaises(NoLongerSupportedException):
+            fetch_attachment = a.fetch_attachment("test")
 
     def testDocumentDeleteAttachment(self):
         db = self.server.create_db('couchdbkit_test')
@@ -588,12 +565,11 @@ class DocumentTestCase(unittest.TestCase):
 
         text_attachment = "un texte attaché"
 
-        a.put_attachment(text_attachment, "test", "text/plain")
-        a.delete_attachment('test')
-        self.assertRaises(ResourceNotFound, a.fetch_attachment, 'test')
-        self.assertFalse('test' in a._doc['_attachments'])
-
-        self.server.delete_db('couchdbkit_test')
+        with self.assertRaises(NoLongerSupportedException):
+            a.put_attachment(text_attachment, "test", "text/plain")
+        with self.assertRaises(NoLongerSupportedException):
+            a.delete_attachment('test')
+        self.assertRaises(NoLongerSupportedException, a.fetch_attachment, 'test')
 
     def testGetOrCreate(self):
         self.server.create_db('couchdbkit_test')
@@ -629,7 +605,7 @@ class DocumentTestCase(unittest.TestCase):
 
         db.bulk_delete([doc1, doc2, doc3])
 
-        print list(db.all_docs(include_docs=True))
+        print(list(db.all_docs(include_docs=True)))
         self.assert_(len(db) == 0)
         self.assert_(db.info()['doc_del_count'] == 3)
 
@@ -731,7 +707,7 @@ class PropertyTestCase(unittest.TestCase):
         test_dates = [
             ([2008, 11, 10, 8, 0, 0], "2008-11-10T08:00:00Z"),
             ([9999, 12, 31, 23, 59, 59], '9999-12-31T23:59:59Z'),
-            ([0001, 1, 1, 0, 0, 1], '0001-01-01T00:00:01Z'),
+            ([1, 1, 1, 0, 0, 1], '0001-01-01T00:00:01Z'),
 
         ]
         for date, date_str in test_dates:
@@ -740,8 +716,6 @@ class PropertyTestCase(unittest.TestCase):
             value = test.field
             self.assert_(isinstance(value, datetime.datetime))
 
-
-
     def testDateProperty(self):
         class Test(Document):
             field = DateProperty()
@@ -756,7 +730,6 @@ class PropertyTestCase(unittest.TestCase):
         value = test.field
         self.assert_(isinstance(value, datetime.date))
 
-
     def testTimeProperty(self):
         class Test(Document):
             field = TimeProperty()
@@ -782,7 +755,7 @@ class PropertyTestCase(unittest.TestCase):
         self.assert_(test._doc['field'] == "test")
         self.assert_(test._doc['field1'] == "2008-11-10T08:00:00Z")
 
-        self.assert_(isinstance(test.field, basestring))
+        self.assert_(isinstance(test.field, six.string_types))
         self.assert_(isinstance(test.field1, datetime.datetime))
         Test._db = self.db
         test.save()
@@ -790,7 +763,7 @@ class PropertyTestCase(unittest.TestCase):
 
         v = doc2.field
         v1 = doc2.field1
-        self.assert_(isinstance(v, basestring))
+        self.assert_(isinstance(v, six.string_types))
         self.assert_(isinstance(v1, datetime.datetime))
 
     def testMixDynamicProperties(self):
@@ -812,8 +785,7 @@ class PropertyTestCase(unittest.TestCase):
         vd = doc2.dynamic_field
 
         self.assert_(isinstance(v1, datetime.datetime))
-        self.assert_(isinstance(vd, basestring))
-
+        self.assert_(isinstance(vd, six.string_types))
 
     def testSchemaProperty1(self):
         class MySchema(DocumentSchema):
@@ -838,7 +810,6 @@ class PropertyTestCase(unittest.TestCase):
         self.assert_(doc2.schema.astring == u"test")
         self.assert_(doc2._doc['schema']['astring'] == u"test")
 
-
     def testSchemaPropertyWithRequired(self):
         class B( Document ):
             class b_schema(DocumentSchema):
@@ -931,7 +902,7 @@ class PropertyTestCase(unittest.TestCase):
 
     def testSchemaWithPythonTypes(self):
         class A(Document):
-            c = unicode()
+            c = six.text_type()
             i = int(4)
         a = A()
         self.assert_(a._doc == {'c': u'', 'doc_type': 'A', 'i': 4})
@@ -1067,7 +1038,6 @@ class PropertyTestCase(unittest.TestCase):
         self.assert_(len(b1.slm) == 2)
         self.assert_(b1.slm[0].s == "test")
 
-
     def testSchemaListPropertySlice(self):
         """SchemaListProperty slice methods
         """
@@ -1089,8 +1059,8 @@ class PropertyTestCase(unittest.TestCase):
         self.assertEqual([b.slm[0].s, b.slm[1].s], [a1.s, a2.s])
         self.assertEqual(b._doc, {
             'doc_type': 'B',
-            'slm': [{'doc_type': 'A', 's': unicode(a1.s)},
-                    {'doc_type': 'A', 's': unicode(a2.s)}]
+            'slm': [{'doc_type': 'A', 's': six.text_type(a1.s)},
+                    {'doc_type': 'A', 's': six.text_type(a2.s)}]
         })
         b.slm.append(a3)
         c = b.slm[1:3]
@@ -1101,10 +1071,9 @@ class PropertyTestCase(unittest.TestCase):
         self.assertEqual(b.slm[0].s, a1.s)
         self.assertEqual(b._doc, {
             'doc_type': 'B',
-            'slm': [{'doc_type': 'A', 's': unicode(a1.s)}]
+            'slm': [{'doc_type': 'A', 's': six.text_type(a1.s)}]
         })
 
-
     def testSchemaListPropertyContains(self):
         """SchemaListProperty contains method
         """
@@ -1123,7 +1092,6 @@ class PropertyTestCase(unittest.TestCase):
         self.assertTrue(a1 in b.slm)
         self.assertFalse(a2 in b.slm)
 
-
     def testSchemaListPropertyCount(self):
         """SchemaListProperty count method
         """
@@ -1141,7 +1109,6 @@ class PropertyTestCase(unittest.TestCase):
         b.slm = [a1, a2, a1]
         self.assertEqual(b.slm.count(a1), 2)
 
-
     def testSchemaListPropertyExtend(self):
         """SchemaListProperty extend method
         """
@@ -1161,8 +1128,8 @@ class PropertyTestCase(unittest.TestCase):
         self.assertEqual([b.slm[0].s, b.slm[1].s], [a1.s, a2.s])
         self.assertEqual(b._doc, {
             'doc_type': 'B',
-            'slm': [{'doc_type': 'A', 's': unicode(a1.s)},
-                    {'doc_type': 'A', 's': unicode(a2.s)}]
+            'slm': [{'doc_type': 'A', 's': six.text_type(a1.s)},
+                    {'doc_type': 'A', 's': six.text_type(a2.s)}]
         })
 
     def testSchemaListPropertyIndex(self):
@@ -1216,12 +1183,11 @@ class PropertyTestCase(unittest.TestCase):
             [b.slm[0].s, b.slm[1].s, b.slm[2].s], [a1.s, a2.s, a3.s])
         self.assertEqual(b._doc, {
             'doc_type': 'B',
-            'slm': [{'doc_type': 'A', 's': unicode(a1.s)},
-                    {'doc_type': 'A', 's': unicode(a2.s)},
-                    {'doc_type': 'A', 's': unicode(a3.s)}]
+            'slm': [{'doc_type': 'A', 's': six.text_type(a1.s)},
+                    {'doc_type': 'A', 's': six.text_type(a2.s)},
+                    {'doc_type': 'A', 's': six.text_type(a3.s)}]
         })
 
-
     def testSchemaListPropertyPop(self):
         """SchemaListProperty pop method
         """
@@ -1245,8 +1211,8 @@ class PropertyTestCase(unittest.TestCase):
         self.assertEqual([b.slm[0].s, b.slm[1].s], [a1.s, a2.s])
         self.assertEqual(b._doc, {
             'doc_type': 'B',
-            'slm': [{'doc_type': 'A', 's': unicode(a1.s)},
-                    {'doc_type': 'A', 's': unicode(a2.s)}]
+            'slm': [{'doc_type': 'A', 's': six.text_type(a1.s)},
+                    {'doc_type': 'A', 's': six.text_type(a2.s)}]
         })
         v = b.slm.pop(0)
         self.assertEqual(v.s, a1.s)
@@ -1254,10 +1220,9 @@ class PropertyTestCase(unittest.TestCase):
         self.assertEqual(b.slm[0].s, a2.s)
         self.assertEqual(b._doc, {
             'doc_type': 'B',
-            'slm': [{'doc_type': 'A', 's': unicode(a2.s)}]
+            'slm': [{'doc_type': 'A', 's': six.text_type(a2.s)}]
         })
 
-
     def testSchemaListPropertyRemove(self):
         """SchemaListProperty remove method
         """
@@ -1278,7 +1243,7 @@ class PropertyTestCase(unittest.TestCase):
         self.assertEqual(b.slm[0].s, a2.s)
         self.assertEqual(b._doc, {
             'doc_type': 'B',
-            'slm': [{'doc_type': 'A', 's': unicode(a2.s)}]
+            'slm': [{'doc_type': 'A', 's': six.text_type(a2.s)}]
         })
         with self.assertRaises(ValueError) as cm:
             b.slm.remove(a1)
@@ -1306,11 +1271,10 @@ class PropertyTestCase(unittest.TestCase):
         self.assertEqual([b.slm[0].s, b.slm[1].s], [a2.s, a1.s])
         self.assertEqual(b._doc, {
             'doc_type': 'B',
-            'slm': [{'doc_type': 'A', 's': unicode(a2.s)},
-                    {'doc_type': 'A', 's': unicode(a1.s)}]
+            'slm': [{'doc_type': 'A', 's': six.text_type(a2.s)},
+                    {'doc_type': 'A', 's': six.text_type(a1.s)}]
         })
 
-
     def testSchemaListPropertySort(self):
         """SchemaListProperty sort method
         """
@@ -1330,24 +1294,26 @@ class PropertyTestCase(unittest.TestCase):
         self.assertEqual([b.slm[0].s, b.slm[1].s], [a1.s, a2.s])
         self.assertEqual(b._doc, {
             'doc_type': 'B',
-            'slm': [{'doc_type': 'A', 's': unicode(a1.s)},
-                    {'doc_type': 'A', 's': unicode(a2.s)}]
+            'slm': [{'doc_type': 'A', 's': six.text_type(a1.s)},
+                    {'doc_type': 'A', 's': six.text_type(a2.s)}]
         })
         b.slm.sort(key=lambda item: item['s'], reverse=True)
         self.assertEqual([b.slm[0].s, b.slm[1].s], [a2.s, a1.s])
         self.assertEqual(b._doc, {
             'doc_type': 'B',
-            'slm': [{'doc_type': 'A', 's': unicode(a2.s)},
-                    {'doc_type': 'A', 's': unicode(a1.s)}]
-        })
-        b.slm.sort(cmp=lambda x, y: cmp(x['s'].lower(), y['s'].lower()))
-        self.assertEqual([b.slm[0].s, b.slm[1].s], [a1.s, a2.s])
-        self.assertEqual(b._doc, {
-            'doc_type': 'B',
-            'slm': [{'doc_type': 'A', 's': unicode(a1.s)},
-                    {'doc_type': 'A', 's': unicode(a2.s)}]
+            'slm': [{'doc_type': 'A', 's': six.text_type(a2.s)},
+                    {'doc_type': 'A', 's': six.text_type(a1.s)}]
         })
 
+        # Only test in Python 2, since sort does not take the `cmp` kwarg in Python 3.
+        if six.PY2:
+            b.slm.sort(cmp=lambda x, y: cmp(x['s'].lower(), y['s'].lower()))
+            self.assertEqual([b.slm[0].s, b.slm[1].s], [a1.s, a2.s])
+            self.assertEqual(b._doc, {
+                'doc_type': 'B',
+                'slm': [{'doc_type': 'A', 's': six.text_type(a1.s)},
+                        {'doc_type': 'A', 's': six.text_type(a2.s)}]
+            })
 
     def testSchemaDictProperty(self):
         class A(DocumentSchema):
@@ -1377,7 +1343,6 @@ class PropertyTestCase(unittest.TestCase):
         self.assert_(b1.d['v1'].i == 123)
         self.assert_(b1.d[23].i == 42)
 
-
     def testListProperty(self):
         from datetime import datetime
         class A(Document):
@@ -1414,9 +1379,10 @@ class PropertyTestCase(unittest.TestCase):
         self.assertRaises(BadValueError, a.save)
         try:
             a.validate()
-        except BadValueError, e:
-            pass
-        self.assert_(str(e) == 'Property l is required.')
+        except BadValueError as e:
+            self.assert_(str(e) == 'Property l is required.')
+        else:
+            self.assert_(False)
 
         d = datetime(2009, 4, 13, 22, 56, 10, 967388)
         a.l.append(d)
@@ -1456,7 +1422,6 @@ class PropertyTestCase(unittest.TestCase):
         b1.ls.remove(u'hello')
         self.assert_(u'hello' not in b1.ls)
 
-
     def testListPropertyExtend(self):
         """list extend method for property w/o type
         """
@@ -1468,7 +1433,6 @@ class PropertyTestCase(unittest.TestCase):
         self.assert_(a.l == [42, 24])
         self.assert_(a._doc == {'doc_type': 'A', 'l': [42, 24]})
 
-
     def testListPropertyExtendWithType(self):
         """list extend method for property w/ type
         """
@@ -1486,7 +1450,6 @@ class PropertyTestCase(unittest.TestCase):
             'l': ['2011-03-11T21:31:01Z', '2011-11-03T13:12:02Z']
         })
 
-
     def testListPropertyInsert(self):
         """list insert method for property w/o type
         """
@@ -1499,7 +1462,6 @@ class PropertyTestCase(unittest.TestCase):
         self.assertEqual(a.l, [42, 4224, 24])
         self.assertEqual(a._doc, {'doc_type': 'A', 'l': [42, 4224, 24]})
 
-
     def testListPropertyInsertWithType(self):
         """list insert method for property w/ type
         """
@@ -1521,7 +1483,6 @@ class PropertyTestCase(unittest.TestCase):
                   '2010-01-12T03:02:03Z']
         })
 
-
     def testListPropertyPop(self):
         """list pop method for property w/o type
         """
@@ -1539,7 +1500,6 @@ class PropertyTestCase(unittest.TestCase):
         self.assert_(a.l == [24])
         self.assert_(a._doc == {'doc_type': 'A', 'l': [24]})
 
-
     def testListPropertyPopWithType(self):
         """list pop method for property w/ type
         """
@@ -1556,7 +1516,6 @@ class PropertyTestCase(unittest.TestCase):
         self.assertEqual(v, d3)
         self.assertEqual(a.l, [d1, d2])
 
-
     def testDictProperty(self):
         from datetime import datetime
         class A(Document):
@@ -1632,9 +1591,10 @@ class PropertyTestCase(unittest.TestCase):
         self.assertRaises(BadValueError, a.save)
         try:
             a.save()
-        except BadValueError, e:
-            pass
-        self.assert_(str(e) == 'Property d is required.')
+        except BadValueError as e:
+            self.assert_(str(e) == 'Property d is required.')
+        else:
+            self.assert_(False)
 
         d = datetime(2009, 4, 13, 22, 56, 10, 967388)
         a.d['date'] = d
@@ -1825,7 +1785,6 @@ class PropertyTestCase(unittest.TestCase):
         self.assert_(b.to_json()['d'] == {})
 
 
-
 if support_setproperty:
     class SetPropertyTestCase(unittest.TestCase):
         def testSetPropertyConstructor(self):
@@ -1844,8 +1803,12 @@ if support_setproperty:
             with self.assertRaises(ValueError) as cm:
                 class C(Document):
                     s = SetProperty(item_type=tuple)
-            self.assertIn(
-                "item_type <type 'tuple'> not in set([", str(cm.exception))
+            if six.PY3:
+                self.assertIn(
+                    "item_type <class 'tuple'> not in ", str(cm.exception))
+            else:
+                self.assertIn(
+                    "item_type <type 'tuple'> not in ", str(cm.exception))
 
 
         def testSetPropertyAssignment(self):
